#include "parse_util.h"
#include "parser.tab.h"

YYSTYPE yylval;

char *token_num_to_string (int token_num) {
  switch (token_num) {
    case T_INTCONST: return "integer constant";
    case T_LPAREN: return "left parenthesis (()";
    case T_RPAREN: return "right parenthesis ())";
    case T_PLUS: return "plus (+)";
    case T_TIMES: return "times (*)";
    case T_SEMI: return "semi-colon (;)";
    case T_EOF: return "end of file";
  }
}

/****************************************************************************
                          Use if next token is
  Stmts -> E ; Stmts      int, (
  Stmts -> ""             $
  
  E -> T EPrime           int, (
  
  EPrime -> ""            ;
  EPrime -> + T EPrime    +
  
  T -> F TPrime           int, (
  
  TPrime -> ""            ;, +
  TPrime -> * F TPrime    *
  
  F -> int                int
  F -> ( E )              (
****************************************************************************/

int next_token;

GeneralNode *do_E();
GeneralNode *do_EPrime(GeneralNode *left);
GeneralNode *do_T();
GeneralNode *do_TPrime(GeneralNode *left);
GeneralNode *do_F();

void match (int token_num) {
  if (token_num == next_token) {
    next_token = yylex();
  }
  else
    throw token_num_to_string(token_num);
}

GeneralNode *do_Stmts () {
  if ((next_token == T_INTCONST) || (next_token == T_LPAREN)) {
    GeneralNode *first = do_E();
    match(T_SEMI);
    GeneralNode *rest = do_Stmts();
    return new StmtsNode(first,rest);
  }
  else if (next_token == T_EOF)
    return 0;
  else
    throw "integer constant, left parenthesis (() or end of file";
}

GeneralNode *do_E () {
  GeneralNode *left = do_T();
  return do_EPrime(left);
}

GeneralNode *do_EPrime(GeneralNode *left) {
  if ((next_token == T_SEMI) || (next_token == T_RPAREN))
    return left;
  else if (next_token == T_PLUS) {
    match(T_PLUS);
    GeneralNode *right = do_T();
    return do_EPrime(new BinaryNode(T_PLUS,left,right));
  }
  else
    throw "semi-colon (;) or plus (+)";
}

GeneralNode *do_T () {
  GeneralNode *left = do_F();
  return do_TPrime(left);
}

GeneralNode *do_TPrime(GeneralNode *left) {
  if ((next_token == T_SEMI) || (next_token == T_PLUS) || (next_token == T_RPAREN))
    return left;
  else if (next_token == T_TIMES) {
    match(T_TIMES);
    GeneralNode *right = do_F();
    return do_TPrime(new BinaryNode(T_TIMES,left,right));
  }
  else
    throw "semi-colon (;), plus (+), or times (*)";
}

GeneralNode *do_F () {
  if (next_token == T_INTCONST) {
    GeneralNode *result = yylval;
    match(T_INTCONST);
    return result;
  }
  else if (next_token == T_LPAREN) {
    match(T_LPAREN);
    GeneralNode *expr = do_E();
    match(T_RPAREN);
    return new UnaryNode(T_LPAREN,expr);
  }
  else
    throw "integer constant or left parenthesis (()";
}

int yyparse () {
  next_token = yylex();
  try {
    parse_root = do_Stmts();
  }
  catch (const char *string) {
    cout << "Error: expected " << string << ", found " << token_num_to_string(next_token) << " at line number " << line_num << " character " << char_num << endl;
    return 1;
  }
  return 0;
}
